#!/usr/bin/python

import optparse
import sys
import os
import stat
import tempfile
import shutil
import types
from socket import gethostname
import signal
import atexit

transferCommand = "/lcg/bin/lcg-cp -b -D srmv2 -v -t 14400"

cleanupDirs = []
gram_scratch = ""

def Execute(program):
    print "Program = %s" % program
    command = ' '.join(program)
    command = "./%s" % command
    if command[len(command)-1] == '"':
        try:
            command = command[:len(command)-1]
            command = command.replace('"', '', 1)
        except:
            print "Error while removing quotes from arguments, check input:\n%s" % ( command )
            return
    print "About to execute: %s" % command
    os.chmod(program[0], stat.S_IXUSR | stat.S_IRUSR | stat.S_IWUSR)
    return os.system(command)


def TransferOuptuts(outputFiles):
    try:
        numOuputs = len(outputFiles)
    except TypeError:
        print "No output files need to be transferred"
        return
    
    print "Detected %i ouptut files, beginning transfers." % (numOuputs)

    for file in outputFiles:
        try:
            origFile, dest = file.split('=', 1)
        except:
            origFile = file
            #print "Malformed output statment: %s\nMust be origfile=dest" % ( file )
            #continue
        
        if os.path.exists(origFile) and (dest != None):
            vdtloc = os.getenv("OSG_GRID")
            command = "source $OSG_GRID/setup.sh; %s%s ./%s %s" % (vdtloc, transferCommand, origFile, dest)
            print "About to execute: %s" % command
            os.system(command)
        elif os.path.exists(origFile):
            command = "cp %s %s" % ( origFile, gram_scratch )
            print "About to execute: %s" % command
            if os.system(command) != 0:
                print "Command failed!: %s" % command
        else:
            print "Output file not found: %s" % origFile




def TransferInputs(inputFiles):
    try:
        numInputs = len(inputFiles)
    except TypeError:
        print "No input files need to be transferred"
        return
    
    print "Detected %i input files, beginning transfers." % ( numInputs)
    

    for file in inputFiles:
        try:
            splits = file.split('=')
            src = '='.join(splits[:len(splits)-1])
            resultFile = splits[len(splits)-1]
        except:
            print "Malformed input statment: %s\nMust be src=filename" % ( file )
            continue

        vdtloc = os.getenv("OSG_GRID")
        command = "source $OSG_GRID/setup.sh; %s%s %s %s" % (vdtloc, transferCommand, src, resultFile)
        print "Transferring %s" % (file)
        print "About to execute: %s" % (command)
        os.system(command)
        if os.path.exists(resultFile):
            print "Transfer successful :)"
        else:
            print "Transfer unsucessful :("

def PrintDebugInfo():
    print "Running on %s %s" % (gethostname(), os.getenv("OSG_SITE_NAME"))
    os.system("uname -a")
    if os.path.exists("/etc/redhat-release"):
        os.system("cat /etc/redhat-release")
    os.system("ulimit -a")
    os.system("env")
    os.system("cat /proc/cpuinfo")
    os.system("cat /proc/meminfo")

    print "Initial working directory: %s" % os.getcwd()
    print "Contents of directory:"
    os.system("ls -lhA")

    print
    print "------------------------------------------"
    print


def CreateTmpDir(opts):
    gram_scratch = os.getcwd()
    tmpTarget = os.getenv("OSG_WN_TMP")
    if tmpTarget is None:
        tmpDir = tempfile.mkdtemp(prefix=opts.vo)
    else:
        tmpDir = tempfile.mkdtemp(dir=tmpTarget, prefix=opts.vo)

    cleanupDirs.append(tmpDir)
    print "Created working directory: %s" % tmpDir
    
    command = "cp * %s" % tmpDir
    print "Copying files to tmp dir: %s" % command
    os.system(command)
    os.chdir(tmpDir)


def Cleanup():
    print "Cleaining up..."
    for dir in cleanupDirs:
        print "Deleting directory: %s" % dir
        try:
            shutil.rmtree(dir)
        except:
            print "Error deleting %s." % dir

def AddOptions(parser):
    parser.add_option("-i", "--input", action="append", dest="inputFiles", help="input files to be sent to sandbox")
    parser.add_option("-o", "--output", action="append", dest="outputFiles", help="output files to be staged out")
    parser.add_option("-v", "--vo", dest="vo", default="unknown_vo", help="VO running this jobs.")


def main():
    #atexit.register(Cleanup)
    signal.signal(signal.SIGTERM, Cleanup)
    signal.signal(signal.SIGINT, Cleanup)
    signal.signal(signal.SIGQUIT, Cleanup)
    parser = optparse.OptionParser()
    AddOptions(parser) 
    (opts, args) = parser.parse_args()
    
    PrintDebugInfo()
    #should I change directories to a local worker directory???? YES
    CreateTmpDir(opts)

    #First, transfer input files into the current directory
    TransferInputs(opts.inputFiles)

    #Execute the program
    result = Execute(args)

    # Print out the results of the run.
    try:
        result = int(result)
    except:
        pass
    if isinstance(result, types.IntType) and result == 0:
        print "=== RUN SUCCESSFUL ==="
    else:
        print "Job failed with exit code %i" % result

    #Transfer the outputs
    TransferOuptuts(opts.outputFiles)

    Cleanup()

    sys.exit(result)

if __name__ == "__main__":
    main()


